#!/usr/bin/env python3
import re
from pathlib import Path
from dotenv import dotenv_values
import parsy as ps
from icecream import ic
import os
from pprint import pprint as print
from parsy import regex, seq

# TODO: (akristip) refactor these to use `<<` instead of capture groups
parse_header = regex(
    r"(.*)(^[^\n]*-+\n.*EASYPY JOB START.*?-+\|\n)",
    re.MULTILINE | re.DOTALL,
    1,
)
parse_test_output = regex(
    r"(.*)(^[^\n]*-+\n.*EASYPY JOB END.*?-+\|)",
    re.MULTILINE | re.DOTALL,
    1,
)
parse_post_test = regex(
    r"(.*)(\+-+\+\n\| +Easypy Report +\|\n\+-+\+)",
    re.MULTILINE | re.DOTALL,
    1,
)
parse_report = regex(
    r"(.*)(\+-+\+\n\| +Task Result Summary +\|\n\+-+\+)",
    re.MULTILINE | re.DOTALL,
    1,
)
parse_summary = regex(
    r"(.*)(\+-+\+\n\| +Task Result Details +\|\n\+-+\+)",
    re.MULTILINE | re.DOTALL,
    1,
)

timestamp = regex(r"\d{4}-\d{2}-\d{2}.\d{2}:\d{2}:\d{2}(,\d{3})?")
log_level = regex(r" (.+?) ", 0, 1)
source = regex(r"\[([A-Za-z_]+)\]", 0, 1)
messsage = ps.any_char.until(ps.peek(timestamp) | ps.eof)


parse_details = regex(r".*", re.MULTILINE | re.DOTALL)
parse_sections = seq(
    header=parse_header,
    test_output=parse_test_output,
    post_test=parse_post_test,
    report=parse_report,
    summary=parse_summary,
    details=parse_details,
)
parse_lines = seq(
    ts=timestamp,
    level=log_level,
    src=source,
    msg=messsage.map(lambda x: "".join(x).strip()),
).many()


class Config(dict[str, str]):
    def __init__(self):
        self.dict = dotenv_values(".env")

    def __getitem__(self, key: str, /) -> str:
        try:
            return self.dict.__getitem__(key)
        except KeyError:
            return os.environ[key]

    def __setitem__(self, _key: str, _value: str, /):
        raise NotImplementedError("Config is read-only")

    def __delitem__(self, _: str, /):
        raise NotImplementedError("Config is read-only")

    def __iter__(self):
        return iter(self.dict)

    def __len__(self):
        return len(self.dict)

    def __repr__(self):
        return self.dict.__repr__()


if __name__ == "__main__":
    config = Config()
    with Path(config["LOGFILE"]).open() as f:
        log = f.read()
    sections = parse_sections.parse(log)
    lines = parse_lines.parse(sections["test_output"])
    ic(lines)
